@@ -5,8 +5,6 @@
 //
 // Copyright (c) 2011 Canonical Ltd.
 //
-// Written by Gustavo Niemeyer <gustavo.niemeyer@canonical.com>
-//
 
 package ec2
 
@@ -15,7 +13,6 @@ import (
 	"encoding/hex"
 	"encoding/xml"
 	"fmt"
-	"launchpad.net/goamz/aws"
 	"log"
 	"net/http"
 	"net/http/httputil"
@@ -23,9 +20,21 @@ import (
 	"sort"
 	"strconv"
 	"time"
+
+	"encoding/base64"
+	"launchpad.net/goamz/aws"
 )
 
-const debug = false
+const (
+	debug = false
+
+	// legacyAPIVersion is the AWS API version used for all but
+	// VPC-related requests.
+	legacyAPIVersion = "2011-12-15"
+
+	// AWS API version used for VPC-related calls.
+	vpcAPIVersion = "2013-10-15"
+)
 
 // The EC2 type encapsulates operations with a specific EC2 region.
 type EC2 struct {
@@ -118,22 +127,25 @@ type xmlErrors struct {
 
 var timeNow = time.Now
 
+// resp = response structure that will get inflated by XML unmarshaling.
 func (ec2 *EC2) query(params map[string]string, resp interface{}) error {
-	params["Version"] = "2011-12-15"
-	params["Timestamp"] = timeNow().In(time.UTC).Format(time.RFC3339)
-	endpoint, err := url.Parse(ec2.Region.EC2Endpoint)
+
+	req, err := http.NewRequest("GET", ec2.Region.EC2Endpoint, nil)
 	if err != nil {
 		return err
 	}
-	if endpoint.Path == "" {
-		endpoint.Path = "/"
+
+	// Add the params passed in to the query string
+	query := req.URL.Query()
+	for varName, varVal := range params {
+		query.Add(varName, varVal)
 	}
-	sign(ec2.Auth, "GET", endpoint.Path, params, endpoint.Host)
-	endpoint.RawQuery = multimap(params).Encode()
-	if debug {
-		log.Printf("get { %v } -> {\n", endpoint.String())
-	}
-	r, err := http.Get(endpoint.String())
+	query.Add("Timestamp", timeNow().In(time.UTC).Format(time.RFC3339))
+	req.URL.RawQuery = query.Encode()
+
+	ec2.Region.Sign(req, ec2.Auth)
+
+	r, err := http.DefaultClient.Do(req)
 	if err != nil {
 		return err
 	}
@@ -147,8 +159,7 @@ func (ec2 *EC2) query(params map[string]string, resp interface{}) error {
 	if r.StatusCode != 200 {
 		return buildError(r)
 	}
-	err = xml.NewDecoder(r.Body).Decode(resp)
-	return err
+	return xml.NewDecoder(r.Body).Decode(resp)
 }
 
 func multimap(p map[string]string) url.Values {
@@ -175,8 +186,17 @@ func buildError(r *http.Response) error {
 }
 
 func makeParams(action string) map[string]string {
+	return makeParamsWithVersion(action, legacyAPIVersion)
+}
+
+func makeParamsVPC(action string) map[string]string {
+	return makeParamsWithVersion(action, vpcAPIVersion)
+}
+
+func makeParamsWithVersion(action, version string) map[string]string {
 	params := make(map[string]string)
 	params["Action"] = action
+	params["Version"] = version
 	return params
 }
 
@@ -189,6 +209,37 @@ func addParamsList(params map[string]string, label string, ids []string) {
 // ----------------------------------------------------------------------------
 // Instance management functions and types.
 
+// RunNetworkInterface encapsulates options for a single network
+// interface, specified when calling RunInstances.
+//
+// If Id is set, it must match an existing VPC network interface, and
+// in this case only a single instance can be launched. If Id is not
+// set, a new network interface will be created for each instance.
+//
+// The following fields are required when creating a new network
+// interface (i.e. Id is empty): DeviceIndex, SubnetId, Description
+// (only used if set), SecurityGroupIds.
+//
+// PrivateIPs can be used to add one or more private IP addresses to a
+// network interface. Only one of the IP addresses can be set as
+// primary. If none are given, EC2 selects a primary IP for each
+// created interface from the subnet pool.
+//
+// When SecondaryPrivateIPCount is non-zero, EC2 allocates that number
+// of IP addresses from within the subnet range and sets them as
+// secondary IPs. The number of IP addresses that can be assigned to a
+// network interface varies by instance type.
+type RunNetworkInterface struct {
+	Id                      string
+	DeviceIndex             int
+	SubnetId                string
+	Description             string
+	PrivateIPs              []PrivateIP
+	SecurityGroupIds        []string
+	DeleteOnTermination     bool
+	SecondaryPrivateIPCount int
+}
+
 // The RunInstances type encapsulates options for the respective request in EC2.
 //
 // See http://goo.gl/Mcm3b for more details.
@@ -210,6 +261,7 @@ type RunInstances struct {
 	ShutdownBehavior      string
 	PrivateIPAddress      string
 	BlockDeviceMappings   []BlockDeviceMapping
+	NetworkInterfaces     []RunNetworkInterface
 }
 
 // Response to a RunInstances request.
@@ -227,33 +279,37 @@ type RunInstancesResp struct {
 //
 // See http://goo.gl/OCH8a for more details.
 type Instance struct {
-	InstanceId         string          `xml:"instanceId"`
-	InstanceType       string          `xml:"instanceType"`
-	ImageId            string          `xml:"imageId"`
-	PrivateDNSName     string          `xml:"privateDnsName"`
-	DNSName            string          `xml:"dnsName"`
-	IPAddress          string          `xml:"ipAddress"`
-	PrivateIPAddress   string          `xml:"privateIpAddress"`
-	KeyName            string          `xml:"keyName"`
-	AMILaunchIndex     int             `xml:"amiLaunchIndex"`
-	Hypervisor         string          `xml:"hypervisor"`
-	VirtType           string          `xml:"virtualizationType"`
-	Monitoring         string          `xml:"monitoring>state"`
-	AvailZone          string          `xml:"placement>availabilityZone"`
-	PlacementGroupName string          `xml:"placement>groupName"`
-	State              InstanceState   `xml:"instanceState"`
-	Tags               []Tag           `xml:"tagSet>item"`
-	SecurityGroups     []SecurityGroup `xml:"groupSet>item"`
+	InstanceId         string             `xml:"instanceId"`
+	InstanceType       string             `xml:"instanceType"`
+	ImageId            string             `xml:"imageId"`
+	PrivateDNSName     string             `xml:"privateDnsName"`
+	DNSName            string             `xml:"dnsName"`
+	IPAddress          string             `xml:"ipAddress"`
+	PrivateIPAddress   string             `xml:"privateIpAddress"`
+	SubnetId           string             `xml:"subnetId"`
+	VPCId              string             `xml:"vpcId"`
+	SourceDestCheck    bool               `xml:"sourceDestCheck"`
+	KeyName            string             `xml:"keyName"`
+	AMILaunchIndex     int                `xml:"amiLaunchIndex"`
+	Hypervisor         string             `xml:"hypervisor"`
+	VirtType           string             `xml:"virtualizationType"`
+	Monitoring         string             `xml:"monitoring>state"`
+	AvailZone          string             `xml:"placement>availabilityZone"`
+	PlacementGroupName string             `xml:"placement>groupName"`
+	State              InstanceState      `xml:"instanceState"`
+	Tags               []Tag              `xml:"tagSet>item"`
+	SecurityGroups     []SecurityGroup    `xml:"groupSet>item"`
+	NetworkInterfaces  []NetworkInterface `xml:"networkInterfaceSet>item"`
 }
 
 // RunInstances starts new instances in EC2.
 // If options.MinCount and options.MaxCount are both zero, a single instance
 // will be started; otherwise if options.MaxCount is zero, options.MinCount
-// will be used insteead.
+// will be used instead.
 //
 // See http://goo.gl/Mcm3b for more details.
 func (ec2 *EC2) RunInstances(options *RunInstances) (resp *RunInstancesResp, err error) {
-	params := makeParams("RunInstances")
+	params := prepareRunParams(*options)
 	params["ImageId"] = options.ImageId
 	params["InstanceType"] = options.InstanceType
 	var min, max int
@@ -279,30 +335,8 @@ func (ec2 *EC2) RunInstances(options *RunInstances) (resp *RunInstancesResp, err
 			j++
 		}
 	}
-	for i, b := range options.BlockDeviceMappings {
-		n := strconv.Itoa(i + 1)
-		if b.DeviceName != "" {
-			params["BlockDeviceMapping."+n+".DeviceName"] = b.DeviceName
-		}
-		if b.VirtualName != "" {
-			params["BlockDeviceMapping."+n+".VirtualName"] = b.VirtualName
-		}
-		if b.SnapshotId != "" {
-			params["BlockDeviceMapping."+n+".Ebs.SnapshotId"] = b.SnapshotId
-		}
-		if b.VolumeType != "" {
-			params["BlockDeviceMapping."+n+".Ebs.VolumeType"] = b.VolumeType
-		}
-		if b.VolumeSize > 0 {
-			params["BlockDeviceMapping."+n+".Ebs.VolumeSize"] = strconv.FormatInt(b.VolumeSize, 10)
-		}
-		if b.IOPS > 0 {
-			params["BlockDeviceMapping."+n+".Ebs.Iops"] = strconv.FormatInt(b.IOPS, 10)
-		}
-		if b.DeleteOnTermination {
-			params["BlockDeviceMapping."+n+".Ebs.DeleteOnTermination"] = "true"
-		}
-	}
+	prepareBlockDevices(params, options.BlockDeviceMappings)
+	prepareNetworkInterfaces(params, options.NetworkInterfaces)
 	token, err := clientToken()
 	if err != nil {
 		return nil, err
@@ -319,8 +353,8 @@ func (ec2 *EC2) RunInstances(options *RunInstances) (resp *RunInstancesResp, err
 		params["RamdiskId"] = options.RamdiskId
 	}
 	if options.UserData != nil {
-		userData := make([]byte, b64.EncodedLen(len(options.UserData)))
-		b64.Encode(userData, options.UserData)
+		userData := make([]byte, base64.StdEncoding.EncodedLen(len(options.UserData)))
+		base64.StdEncoding.Encode(userData, options.UserData)
 		params["UserData"] = string(userData)
 	}
 	if options.AvailZone != "" {
@@ -353,6 +387,81 @@ func (ec2 *EC2) RunInstances(options *RunInstances) (resp *RunInstancesResp, err
 	return
 }
 
+func prepareRunParams(options RunInstances) map[string]string {
+	if options.SubnetId != "" || len(options.NetworkInterfaces) > 0 {
+		// When either SubnetId or NetworkInterfaces are specified, we
+		// need to use the API version with complete VPC support.
+		return makeParamsVPC("RunInstances")
+	} else {
+		return makeParams("RunInstances")
+	}
+}
+
+func prepareBlockDevices(params map[string]string, blockDevs []BlockDeviceMapping) {
+	for i, b := range blockDevs {
+		n := strconv.Itoa(i + 1)
+		prefix := "BlockDeviceMapping." + n
+		if b.DeviceName != "" {
+			params[prefix+".DeviceName"] = b.DeviceName
+		}
+		if b.VirtualName != "" {
+			params[prefix+".VirtualName"] = b.VirtualName
+		}
+		if b.SnapshotId != "" {
+			params[prefix+".Ebs.SnapshotId"] = b.SnapshotId
+		}
+		if b.VolumeType != "" {
+			params[prefix+".Ebs.VolumeType"] = b.VolumeType
+		}
+		if b.VolumeSize > 0 {
+			params[prefix+".Ebs.VolumeSize"] = strconv.FormatInt(b.VolumeSize, 10)
+		}
+		if b.IOPS > 0 {
+			params[prefix+".Ebs.Iops"] = strconv.FormatInt(b.IOPS, 10)
+		}
+		if b.DeleteOnTermination {
+			params[prefix+".Ebs.DeleteOnTermination"] = "true"
+		}
+	}
+}
+
+func prepareNetworkInterfaces(params map[string]string, nics []RunNetworkInterface) {
+	for i, ni := range nics {
+		// Unlike other lists, NetworkInterface and PrivateIpAddresses
+		// should start from 0, not 1, according to the examples
+		// requests in the API documentation here http://goo.gl/Mcm3b.
+		n := strconv.Itoa(i)
+		prefix := "NetworkInterface." + n
+		if ni.Id != "" {
+			params[prefix+".NetworkInterfaceId"] = ni.Id
+		}
+		params[prefix+".DeviceIndex"] = strconv.Itoa(ni.DeviceIndex)
+		if ni.SubnetId != "" {
+			params[prefix+".SubnetId"] = ni.SubnetId
+		}
+		if ni.Description != "" {
+			params[prefix+".Description"] = ni.Description
+		}
+		for j, gid := range ni.SecurityGroupIds {
+			k := strconv.Itoa(j + 1)
+			params[prefix+".SecurityGroupId."+k] = gid
+		}
+		if ni.DeleteOnTermination {
+			params[prefix+".DeleteOnTermination"] = "true"
+		}
+		if ni.SecondaryPrivateIPCount > 0 {
+			val := strconv.Itoa(ni.SecondaryPrivateIPCount)
+			params[prefix+".SecondaryPrivateIpAddressCount"] = val
+		}
+		for j, ip := range ni.PrivateIPs {
+			k := strconv.Itoa(j)
+			subprefix := prefix + ".PrivateIpAddresses." + k
+			params[subprefix+".PrivateIpAddress"] = ip.Address
+			params[subprefix+".Primary"] = strconv.FormatBool(ip.IsPrimary)
+		}
+	}
+}
+
 func clientToken() (string, error) {
 	// Maximum EC2 client token size is 64 bytes.
 	// Each byte expands to two when hex encoded.
@@ -621,14 +730,26 @@ type CreateSecurityGroupResp struct {
 	RequestId string `xml:"requestId"`
 }
 
-// CreateSecurityGroup run a CreateSecurityGroup request in EC2, with the provided
-// name and description.
+// CreateSecurityGroup creates a security group with the provided name
+// and description.
 //
 // See http://goo.gl/Eo7Yl for more details.
 func (ec2 *EC2) CreateSecurityGroup(name, description string) (resp *CreateSecurityGroupResp, err error) {
-	params := makeParams("CreateSecurityGroup")
+	return ec2.CreateSecurityGroupVPC("", name, description)
+}
+
+// CreateSecurityGroupVPC creates a security group in EC2, associated
+// with the given VPC ID. If vpcId is empty, this call is equivalent
+// to CreateSecurityGroup.
+//
+// See http://goo.gl/Eo7Yl for more details.
+func (ec2 *EC2) CreateSecurityGroupVPC(vpcId, name, description string) (resp *CreateSecurityGroupResp, err error) {
+	params := makeParamsVPC("CreateSecurityGroup")
 	params["GroupName"] = name
 	params["GroupDescription"] = description
+	if vpcId != "" {
+		params["VpcId"] = vpcId
+	}
 
 	resp = &CreateSecurityGroupResp{}
 	err = ec2.query(params, resp)
@@ -653,6 +774,7 @@ type SecurityGroupsResp struct {
 // See http://goo.gl/CIdyP for more details.
 type SecurityGroupInfo struct {
 	SecurityGroup
+	VPCId       string   `xml:"vpcId"`
 	OwnerId     string   `xml:"ownerId"`
 	Description string   `xml:"groupDescription"`
 	IPPerms     []IPPerm `xml:"ipPermissions>item"`
@@ -803,7 +925,7 @@ func (ec2 *EC2) authOrRevoke(op string, group SecurityGroup, perms []IPPerm) (re
 	return resp, nil
 }
 
-// ResourceTag represents key-value metadata used to classify and organize
+// Tag represents key-value metadata used to classify and organize
 // EC2 instances.
 //
 // See http://goo.gl/bncl3 for more details
@@ -893,3 +1015,77 @@ func (ec2 *EC2) RebootInstances(ids ...string) (resp *SimpleResp, err error) {
 	}
 	return resp, nil
 }
+
+// ----------------------------------------------------------------------------
+// Availability zone management functions and types.
+// See http://goo.gl/ylxT4R for more details.
+
+// AvailabilityZonesResp represents a response to a DescribeAvailabilityZones
+// request in EC2.
+type AvailabilityZonesResp struct {
+	RequestId string                 `xml:"requestId"`
+	Zones     []AvailabilityZoneInfo `xml:"availabilityZoneInfo>item"`
+}
+
+// AvailabilityZoneInfo encapsulates details for an availability zone in EC2.
+type AvailabilityZoneInfo struct {
+	AvailabilityZone
+	State      string   `xml:"zoneState"`
+	MessageSet []string `xml:"messageSet>item"`
+}
+
+// AvailabilityZone represents an EC2 availability zone.
+type AvailabilityZone struct {
+	Name   string `xml:"zoneName"`
+	Region string `xml:"regionName"`
+}
+
+// AvailabilityZones returns details about availability zones in EC2.
+// The filter parameter is optional, and if provided will limit the
+// availability zones returned to those matching the given filtering
+// rules.
+//
+// See http://goo.gl/ylxT4R for more details.
+func (ec2 *EC2) AvailabilityZones(filter *Filter) (resp *AvailabilityZonesResp, err error) {
+	params := makeParams("DescribeAvailabilityZones")
+	filter.addParams(params)
+	resp = &AvailabilityZonesResp{}
+	err = ec2.query(params, resp)
+	if err != nil {
+		return nil, err
+	}
+	return
+}
+
+// AccountAttribute holds information about an account attribute.
+//
+// See http://goo.gl/hBc28j for more details.
+type AccountAttribute struct {
+	Name   string   `xml:"attributeName"`
+	Values []string `xml:"attributeValueSet>item>attributeValue"`
+}
+
+// AccountAttributesResp is the response to an AccountAttributes request.
+//
+// See http://goo.gl/hBc28j for more details.
+type AccountAttributesResp struct {
+	RequestId  string             `xml:"requestId"`
+	Attributes []AccountAttribute `xml:"accountAttributeSet>item"`
+}
+
+// AccountAttributes returns the values of one or more account
+// attributes.
+//
+// See http://goo.gl/hBc28j for more details.
+func (ec2 *EC2) AccountAttributes(attrNames ...string) (*AccountAttributesResp, error) {
+	params := makeParamsVPC("DescribeAccountAttributes")
+	for i, attrName := range attrNames {
+		params["AttributeName."+strconv.Itoa(i+1)] = attrName
+	}
+
+	resp := &AccountAttributesResp{}
+	if err := ec2.query(params, resp); err != nil {
+		return nil, err
+	}
+	return resp, nil
+}
