@@ -9,11 +9,14 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	. "launchpad.net/gocheck"
 	"mime/multipart"
 	"net/http"
 	"net/url"
+	"sort"
 	"strings"
+
+	"gopkg.in/mgo.v2/bson"
+	. "launchpad.net/gocheck"
 )
 
 type TestServerSuite struct {
@@ -43,11 +46,11 @@ func (suite *TestServerSuite) TestNewTestServerReturnsTestServer(c *C) {
 }
 
 func (suite *TestServerSuite) TestGetResourceURI(c *C) {
-	c.Check(getNodeURI("version", "test"), Equals, "/api/version/nodes/test/")
+	c.Check(getNodeURL("0.1", "test"), Equals, "/api/0.1/nodes/test/")
 }
 
 func (suite *TestServerSuite) TestInvalidOperationOnNodesIsBadRequest(c *C) {
-	badURL := getTopLevelNodesURL(suite.server.version) + "?op=procrastinate"
+	badURL := getNodesEndpoint(suite.server.version) + "?op=procrastinate"
 
 	response, err := http.Get(suite.server.Server.URL + badURL)
 	c.Assert(err, IsNil)
@@ -483,6 +486,47 @@ func (suite *TestMAASObjectSuite) TestAcquireOperationGetsRecorded(c *C) {
 	c.Check(value[0], DeepEquals, params)
 }
 
+func (suite *TestMAASObjectSuite) TestNodesRelease(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "mysystemid1"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "mysystemid2"}`)
+	suite.TestMAASObject.TestServer.OwnedNodes()["mysystemid2"] = true
+	nodesObj := suite.TestMAASObject.GetSubObject("nodes/")
+	params := url.Values{"nodes": []string{"mysystemid1", "mysystemid2"}}
+
+	// release should only release mysystemid2, as it is the only one allocated.
+	jsonResponse, err := nodesObj.CallPost("release", params)
+	c.Assert(err, IsNil)
+	releasedNodes, err := jsonResponse.GetArray()
+	c.Assert(err, IsNil)
+	c.Assert(releasedNodes, HasLen, 1)
+	releasedNode, err := releasedNodes[0].GetMAASObject()
+	c.Assert(err, IsNil)
+	systemId, err := releasedNode.GetField("system_id")
+	c.Assert(err, IsNil)
+	c.Assert(systemId, Equals, "mysystemid2")
+
+	// The 'release' operation has been recorded.
+	nodesOperations := suite.TestMAASObject.TestServer.NodesOperations()
+	c.Check(nodesOperations, DeepEquals, []string{"release"})
+	nodesOperationRequestValues := suite.TestMAASObject.TestServer.NodesOperationRequestValues()
+	expectedValues := make(url.Values)
+	expectedValues.Add("nodes", "mysystemid1")
+	expectedValues.Add("nodes", "mysystemid2")
+	c.Check(nodesOperationRequestValues, DeepEquals, []url.Values{expectedValues})
+}
+
+func (suite *TestMAASObjectSuite) TestNodesReleaseUnknown(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "mysystemid"}`)
+	suite.TestMAASObject.TestServer.OwnedNodes()["mysystemid"] = true
+	nodesObj := suite.TestMAASObject.GetSubObject("nodes/")
+	params := url.Values{"nodes": []string{"mysystemid", "what"}}
+
+	// if there are any unknown nodes, none are released.
+	_, err := nodesObj.CallPost("release", params)
+	c.Assert(err, ErrorMatches, `gomaasapi: got error back from server: 400 Bad Request \(Unknown node\(s\): what.\)`)
+	c.Assert(suite.TestMAASObject.TestServer.OwnedNodes()["mysystemid"], Equals, true)
+}
+
 func (suite *TestMAASObjectSuite) TestUploadFile(c *C) {
 	const filename = "myfile.txt"
 	const fileContent = "uploaded contents"
@@ -571,3 +615,233 @@ func (suite *TestMAASObjectSuite) TestReleaseNodeReleasesAcquiredNode(c *C) {
 	_, owned := suite.TestMAASObject.TestServer.OwnedNodes()[systemID]
 	c.Check(owned, Equals, false)
 }
+
+func (suite *TestMAASObjectSuite) TestGetNetworks(c *C) {
+	nodeJSON := `{"system_id": "mysystemid"}`
+	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
+	networkJSON := `{"name": "mynetworkname"}`
+	suite.TestMAASObject.TestServer.NewNetwork(networkJSON)
+	suite.TestMAASObject.TestServer.ConnectNodeToNetwork("mysystemid", "mynetworkname")
+
+	networkMethod := suite.TestMAASObject.GetSubObject("networks")
+	params := url.Values{"node": []string{"mysystemid"}}
+	listNetworkObjects, err := networkMethod.CallGet("", params)
+	c.Assert(err, IsNil)
+
+	networkJSONArray, err := listNetworkObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(networkJSONArray, HasLen, 1)
+
+	listNetworks, err := networkJSONArray[0].GetMAASObject()
+	c.Assert(err, IsNil)
+
+	networkName, err := listNetworks.GetField("name")
+	c.Assert(err, IsNil)
+	c.Check(networkName, Equals, "mynetworkname")
+}
+
+func (suite *TestMAASObjectSuite) TestGetNetworksNone(c *C) {
+	nodeJSON := `{"system_id": "mysystemid"}`
+	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
+
+	networkMethod := suite.TestMAASObject.GetSubObject("networks")
+	params := url.Values{"node": []string{"mysystemid"}}
+	listNetworkObjects, err := networkMethod.CallGet("", params)
+	c.Assert(err, IsNil)
+
+	networkJSONArray, err := listNetworkObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(networkJSONArray, HasLen, 0)
+}
+
+func (suite *TestMAASObjectSuite) TestListNodesWithNetworks(c *C) {
+	nodeJSON := `{"system_id": "mysystemid"}`
+	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
+	networkJSON := `{"name": "mynetworkname"}`
+	suite.TestMAASObject.TestServer.NewNetwork(networkJSON)
+	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("mysystemid", "mynetworkname", "aa:bb:cc:dd:ee:ff")
+
+	nodeListing := suite.TestMAASObject.GetSubObject("nodes")
+	listNodeObjects, err := nodeListing.CallGet("list", url.Values{})
+	c.Assert(err, IsNil)
+
+	listNodes, err := listNodeObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(listNodes, HasLen, 1)
+
+	node, err := listNodes[0].GetMAASObject()
+	c.Assert(err, IsNil)
+	systemId, err := node.GetField("system_id")
+	c.Assert(err, IsNil)
+	c.Check(systemId, Equals, "mysystemid")
+
+	gotResourceURI, err := node.GetField(resourceURI)
+	c.Assert(err, IsNil)
+	apiVersion := suite.TestMAASObject.TestServer.version
+	expectedResourceURI := fmt.Sprintf("/api/%s/nodes/mysystemid/", apiVersion)
+	c.Check(gotResourceURI, Equals, expectedResourceURI)
+
+	macAddressSet, err := node.GetMap()["macaddress_set"].GetArray()
+	c.Assert(err, IsNil)
+	c.Check(macAddressSet, HasLen, 1)
+
+	macAddress, err := macAddressSet[0].GetMap()
+	c.Assert(err, IsNil)
+	macAddressString, err := macAddress["mac_address"].GetString()
+	c.Check(macAddressString, Equals, "aa:bb:cc:dd:ee:ff")
+
+	gotResourceURI, err = macAddress[resourceURI].GetString()
+	c.Assert(err, IsNil)
+	expectedResourceURI = fmt.Sprintf("/api/%s/nodes/mysystemid/macs/%s/", apiVersion, url.QueryEscape("aa:bb:cc:dd:ee:ff"))
+	c.Check(gotResourceURI, Equals, expectedResourceURI)
+}
+
+func (suite *TestMAASObjectSuite) TestListNetworkConnectedMACAddresses(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "node_1"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "node_2"}`)
+	suite.TestMAASObject.TestServer.NewNetwork(`{"name": "net_1"}`)
+	suite.TestMAASObject.TestServer.NewNetwork(`{"name": "net_2"}`)
+	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_2", "net_2", "aa:bb:cc:dd:ee:22")
+	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_1", "net_1", "aa:bb:cc:dd:ee:11")
+	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_2", "net_1", "aa:bb:cc:dd:ee:21")
+	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_1", "net_2", "aa:bb:cc:dd:ee:12")
+
+	nodeListing := suite.TestMAASObject.GetSubObject("networks").GetSubObject("net_1")
+	listNodeObjects, err := nodeListing.CallGet("list_connected_macs", url.Values{})
+	c.Assert(err, IsNil)
+
+	listNodes, err := listNodeObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(listNodes, HasLen, 2)
+
+	node, err := listNodes[0].GetMAASObject()
+	c.Assert(err, IsNil)
+	macAddress, err := node.GetField("mac_address")
+	c.Assert(err, IsNil)
+	c.Check(macAddress == "aa:bb:cc:dd:ee:11" || macAddress == "aa:bb:cc:dd:ee:21", Equals, true)
+	node1_idx := 0
+	if macAddress == "aa:bb:cc:dd:ee:21" {
+		node1_idx = 1
+	}
+
+	node, err = listNodes[node1_idx].GetMAASObject()
+	c.Assert(err, IsNil)
+	macAddress, err = node.GetField("mac_address")
+	c.Assert(err, IsNil)
+	c.Check(macAddress, Equals, "aa:bb:cc:dd:ee:11")
+	nodeResourceURI, err := node.GetField(resourceURI)
+	c.Assert(err, IsNil)
+	apiVersion := suite.TestMAASObject.TestServer.version
+	expectedResourceURI := fmt.Sprintf("/api/%s/nodes/node_1/macs/%s/", apiVersion, url.QueryEscape("aa:bb:cc:dd:ee:11"))
+	c.Check(nodeResourceURI, Equals, expectedResourceURI)
+
+	node, err = listNodes[1-node1_idx].GetMAASObject()
+	c.Assert(err, IsNil)
+	macAddress, err = node.GetField("mac_address")
+	c.Assert(err, IsNil)
+	c.Check(macAddress, Equals, "aa:bb:cc:dd:ee:21")
+	nodeResourceURI, err = node.GetField(resourceURI)
+	c.Assert(err, IsNil)
+	expectedResourceURI = fmt.Sprintf("/api/%s/nodes/node_2/macs/%s/", apiVersion, url.QueryEscape("aa:bb:cc:dd:ee:21"))
+	c.Check(nodeResourceURI, Equals, expectedResourceURI)
+}
+
+func (suite *TestMAASObjectSuite) TestGetVersion(c *C) {
+	networkMethod := suite.TestMAASObject.GetSubObject("version")
+	params := url.Values{"node": []string{"mysystemid"}}
+	versionObject, err := networkMethod.CallGet("", params)
+	c.Assert(err, IsNil)
+
+	versionMap, err := versionObject.GetMap()
+	c.Assert(err, IsNil)
+	jsonArray, ok := versionMap["capabilities"]
+	c.Check(ok, Equals, true)
+	capArray, err := jsonArray.GetArray()
+	for _, capJSONName := range capArray {
+		capName, err := capJSONName.GetString()
+		c.Assert(err, IsNil)
+		c.Check(capName, Equals, "networks-management")
+	}
+}
+
+const nodeDetailsXML = `<?xml version="1.0" standalone="yes" ?>
+<list>
+<node id="node2" claimed="true" class="system" handle="DMI:0001">
+ <description>Computer</description>
+</node>
+</list>`
+
+func (suite *TestMAASObjectSuite) TestNodeDetails(c *C) {
+	nodeJSON := `{"system_id": "mysystemid"}`
+	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
+	suite.TestMAASObject.TestServer.AddNodeDetails("mysystemid", nodeDetailsXML)
+
+	obj := suite.TestMAASObject.GetSubObject("nodes").GetSubObject("mysystemid")
+	uri := obj.URI()
+	result, err := obj.client.Get(uri, "details", nil)
+	c.Assert(err, IsNil)
+
+	bsonObj := map[string]interface{}{}
+	err = bson.Unmarshal(result, &bsonObj)
+	c.Assert(err, IsNil)
+
+	_, ok := bsonObj["lldp"]
+	c.Check(ok, Equals, true)
+	gotXMLText, ok := bsonObj["lshw"]
+	c.Check(ok, Equals, true)
+	c.Check(string(gotXMLText.([]byte)), Equals, string(nodeDetailsXML))
+}
+
+func (suite *TestMAASObjectSuite) TestListNodegroups(c *C) {
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-0", `{"architecture": "arm64", "release": "trusty"}`)
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-1", `{"architecture": "amd64", "release": "precise"}`)
+
+	nodegroupListing := suite.TestMAASObject.GetSubObject("nodegroups")
+	result, err := nodegroupListing.CallGet("list", nil)
+	c.Assert(err, IsNil)
+
+	nodegroups, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(nodegroups, HasLen, 2)
+
+	for _, obj := range nodegroups {
+		nodegroup, err := obj.GetMAASObject()
+		c.Assert(err, IsNil)
+		uuid, err := nodegroup.GetField("uuid")
+		c.Assert(err, IsNil)
+
+		nodegroupResourceURI, err := nodegroup.GetField(resourceURI)
+		c.Assert(err, IsNil)
+		apiVersion := suite.TestMAASObject.TestServer.version
+		expectedResourceURI := fmt.Sprintf("/api/%s/nodegroups/%s/", apiVersion, uuid)
+		c.Check(nodegroupResourceURI, Equals, expectedResourceURI)
+	}
+}
+
+func (suite *TestMAASObjectSuite) TestListBootImages(c *C) {
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-0", `{"architecture": "arm64", "release": "trusty"}`)
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-1", `{"architecture": "amd64", "release": "precise"}`)
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-1", `{"architecture": "ppc64el", "release": "precise"}`)
+
+	bootImageListing := suite.TestMAASObject.GetSubObject("nodegroups").GetSubObject("uuid-1").GetSubObject("boot-images")
+	result, err := bootImageListing.CallGet("", nil)
+	c.Assert(err, IsNil)
+
+	bootImageObjects, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(bootImageObjects, HasLen, 2)
+
+	expectedBootImages := []string{"amd64.precise", "ppc64el.precise"}
+	bootImages := make([]string, len(bootImageObjects))
+	for i, obj := range bootImageObjects {
+		bootimage, err := obj.GetMap()
+		c.Assert(err, IsNil)
+		architecture, err := bootimage["architecture"].GetString()
+		c.Assert(err, IsNil)
+		release, err := bootimage["release"].GetString()
+		c.Assert(err, IsNil)
+		bootImages[i] = fmt.Sprintf("%s.%s", architecture, release)
+	}
+	sort.Strings(bootImages)
+	c.Assert(bootImages, DeepEquals, expectedBootImages)
+}
