@@ -222,7 +222,9 @@ func (api *ManagementAPI) CheckHostedServiceNameAvailability(name string) error
 // DeleteHostedService deletes the named hosted service.
 // See http://msdn.microsoft.com/en-us/library/windowsazure/gg441305.aspx
 func (api *ManagementAPI) DeleteHostedService(serviceName string) error {
-    response, err := api.session.delete("services/hostedservices/"+serviceName, "2010-10-28")
+    url := "services/hostedservices/" + serviceName
+    url = addURLQueryParams(url, "comp", "media")
+    response, err := api.session.delete(url, "2013-08-01")
     if err != nil {
         if IsNotFoundError(err) {
             return nil
@@ -474,6 +476,57 @@ func (api *ManagementAPI) UpdateRole(request *UpdateRoleRequest) error {
     return api.blockUntilCompleted(response)
 }
 
+type DeleteRoleRequest struct {
+    ServiceName    string
+    DeploymentName string
+    RoleName       string
+    DeleteMedia    bool
+}
+
+// DeleteRole deletes a named Role from within the specified Cloud Service
+// and Deployment.
+// See http://msdn.microsoft.com/en-us/library/windowsazure/jj157187.aspx
+func (api *ManagementAPI) DeleteRole(request *DeleteRoleRequest) error {
+    checkPathComponents(request.ServiceName, request.DeploymentName, request.RoleName)
+    url := ("services/hostedservices/" + request.ServiceName +
+        "/deployments/" + request.DeploymentName + "/roles/" + request.RoleName)
+    if request.DeleteMedia {
+        url = addURLQueryParams(url, "comp", "media")
+    }
+    response, err := api.session.delete(url, "2013-08-01")
+    if err != nil {
+        if IsNotFoundError(err) {
+            return nil
+        }
+        return err
+    }
+    return api.blockUntilCompleted(response)
+}
+
+type AddRoleRequest struct {
+    ServiceName      string
+    DeploymentName   string
+    PersistentVMRole *PersistentVMRole
+}
+
+// AddRole creates a new Role within the specified Cloud Service
+// and Deployment.
+// See http://msdn.microsoft.com/en-us/library/windowsazure/jj157187.aspx
+func (api *ManagementAPI) AddRole(request *AddRoleRequest) error {
+    checkPathComponents(request.ServiceName, request.DeploymentName)
+    url := ("services/hostedservices/" + request.ServiceName +
+        "/deployments/" + request.DeploymentName + "/roles")
+    role, err := request.PersistentVMRole.Serialize()
+    if err != nil {
+        return err
+    }
+    response, err := api.session.post(url, "2013-10-01", []byte(role), "application/xml")
+    if err != nil {
+        return err
+    }
+    return api.blockUntilCompleted(response)
+}
+
 type CreateAffinityGroupRequest struct {
     CreateAffinityGroup *CreateAffinityGroup
 }
@@ -540,7 +593,7 @@ func (api *ManagementAPI) DeleteAffinityGroup(request *DeleteAffinityGroupReques
 // be nil.
 // See http://msdn.microsoft.com/en-us/library/windowsazure/jj157196.aspx
 func (api *ManagementAPI) GetNetworkConfiguration() (*NetworkConfiguration, error) {
-    response, err := api.session.get("services/networking/media", "2013-10-01")
+    response, err := api.session.get("services/networking/media", "2012-03-01")
     if err != nil {
         if IsNotFoundError(err) {
             return nil, nil
@@ -565,8 +618,9 @@ func (api *ManagementAPI) SetNetworkConfiguration(cfg *NetworkConfiguration) err
         return err
     }
     response, err := api.session.put(
-        "services/networking/media", "2013-10-01", []byte(body),
-        "application/octet-stream")
+        "services/networking/media", "2012-03-01", []byte(body),
+        "text/plain",
+    )
     if err != nil {
         return err
     }
