@@ -219,6 +219,31 @@ func (*x509DispatcherSuite) TestRequestsFollowRedirects(c *C) {
     }
 }
 
+func (*x509DispatcherSuite) TestRedirectRewindsBody(c *C) {
+    httpRequests := make(chan *Request, 2)
+    serverConflict := makeRecordingHTTPServer(httpRequests, http.StatusConflict, nil, nil)
+    defer serverConflict.Close()
+    redirPath := "/else/where"
+    responseHeaders := make(http.Header)
+    responseHeaders.Set("Location", serverConflict.URL+redirPath)
+    serverRedir := makeRecordingHTTPServer(httpRequests, http.StatusTemporaryRedirect, nil, responseHeaders)
+    defer serverRedir.Close()
+    session, err := newX509Session("subscriptionid", "", "West US", NoRetryPolicy)
+    c.Assert(err, IsNil)
+    path := "/foo/bar"
+    version := "test-version"
+    content := []byte("ponies")
+    contentType := "text/plain"
+
+    request := newX509RequestPOST(serverRedir.URL+path, version, content, contentType)
+    response, err := performX509Request(session, request)
+    c.Assert(err, IsNil)
+    c.Assert(response.StatusCode, Equals, http.StatusConflict)
+    c.Assert(httpRequests, HasLen, 2)
+    c.Assert((<-httpRequests).BodyContent, DeepEquals, content)
+    c.Assert((<-httpRequests).BodyContent, DeepEquals, content)
+}
+
 func (*x509DispatcherSuite) TestRequestsLimitRedirects(c *C) {
     httpRequests := make(chan *Request, 10)
     serverRedir := makeRecordingHTTPServer(httpRequests, http.StatusTemporaryRedirect, nil, nil)
