@@ -291,7 +291,7 @@ func (suite *xmlSuite) TestPersistentVMRoleDeserialize(c *C) {
                 MediaLink:           "path-to-vhd",
             },
         },
-        OSVirtualHardDisk: OSVirtualHardDisk{
+        OSVirtualHardDisk: &OSVirtualHardDisk{
             HostCaching:     "host-caching-mode-of-os-disk",
             DiskName:        "name-of-os-disk",
             MediaLink:       "path-to-vhd",
@@ -360,7 +360,7 @@ func (suite *xmlSuite) TestPersistentVMRoleSerialize(c *C) {
                 MediaLink:           "path-to-vhd",
             },
         },
-        OSVirtualHardDisk: OSVirtualHardDisk{
+        OSVirtualHardDisk: &OSVirtualHardDisk{
             HostCaching:     "host-caching-mode-of-os-disk",
             DiskName:        "name-of-os-disk",
             MediaLink:       "path-to-vhd",
@@ -922,7 +922,8 @@ func (suite *xmlSuite) TestDeploymentWRTGetDeployment(c *C) {
         UpgradeDomainCount: "number-of-upgrade-domains-in-deployment",
         RoleList: []Role{
             {
-                RoleName: "name-of-role",
+                RoleName:  "name-of-role",
+                OsVersion: "operating-system-version",
                 ConfigurationSets: []ConfigurationSet{
                     {
                         ConfigurationSetType:             "LinuxProvisioningConfiguration",
@@ -931,8 +932,9 @@ func (suite *xmlSuite) TestDeploymentWRTGetDeployment(c *C) {
                 },
             },
             {
-                RoleName: "name-of-role",
-                RoleType: "PersistentVMRole",
+                RoleName:  "name-of-role",
+                OsVersion: "operating-system-version",
+                RoleType:  "PersistentVMRole",
                 ConfigurationSets: []ConfigurationSet{
                     {
                         ConfigurationSetType: CONFIG_SET_NETWORK,
@@ -947,15 +949,23 @@ func (suite *xmlSuite) TestDeploymentWRTGetDeployment(c *C) {
                         SubnetNames: &[]string{"name-of-subnet"},
                     },
                 },
-                OSVirtualHardDisk: []OSVirtualHardDisk{
+                AvailabilitySetName: "name-of-availability-set",
+                DataVirtualHardDisks: &[]DataVirtualHardDisk{
                     {
-                        HostCaching:     "host-caching-mode-of-os-disk",
-                        DiskName:        "name-of-os-disk",
-                        MediaLink:       "path-to-vhd",
-                        SourceImageName: "image-used-to-create-os-disk",
-                        OS:              "operating-system-on-os-disk",
+                        HostCaching:         "host-caching-mode-of-data-disk",
+                        DiskName:            "name-of-data-disk",
+                        LUN:                 "logical-unit-number-of-data-disk",
+                        LogicalDiskSizeInGB: "size-of-data-disk",
+                        MediaLink:           "path-to-vhd",
                     },
                 },
+                OSVirtualHardDisk: &OSVirtualHardDisk{
+                    HostCaching:     "host-caching-mode-of-os-disk",
+                    DiskName:        "name-of-os-disk",
+                    MediaLink:       "path-to-vhd",
+                    SourceImageName: "image-used-to-create-os-disk",
+                    OS:              "operating-system-on-os-disk",
+                },
                 RoleSize: "size-of-instance",
             },
         },
@@ -1030,7 +1040,7 @@ func (suite *xmlSuite) TestNewDeploymentForCreateVMDeployment(c *C) {
     deploymentSlot := "staging"
     label := "deploymentLabel"
     vhd := NewOSVirtualHardDisk("hostCaching", "diskLabel", "diskName", "mediaLink", "sourceImageName", "os")
-    roles := []Role{*NewRole("size", "name", []ConfigurationSet{}, []OSVirtualHardDisk{*vhd})}
+    roles := []Role{*NewRole("size", "name", vhd, []ConfigurationSet{})}
     virtualNetworkName := "network"
 
     deployment := NewDeploymentForCreateVMDeployment(name, deploymentSlot, label, roles, virtualNetworkName)
@@ -1303,7 +1313,7 @@ func (suite *xmlSuite) TestNewRole(c *C) {
     configset := []ConfigurationSet{*config}
     vhd := NewOSVirtualHardDisk("hostCaching", "diskLabel", "diskName", "mediaLink", "sourceImageName", "os")
 
-    role := NewRole(rolesize, rolename, configset, []OSVirtualHardDisk{*vhd})
+    role := NewRole(rolesize, rolename, vhd, configset)
     c.Check(role.RoleSize, Equals, rolesize)
     c.Check(role.RoleName, Equals, rolename)
     c.Check(role.ConfigurationSets, DeepEquals, configset)
